<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Magento\Framework\Exception\CouldNotSaveException;
/*
 * Save Additional Data when saving Order Items by Repository 
 */
class ItemSave
{
    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\GiftMessage\Api\OrderItemRepositoryInterface */
    protected $orderItemRepository;

    /**
     * Init plugin
     *
     * @param \Magento\GiftMessage\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\GiftMessage\Api\OrderItemRepositoryInterface $orderItemRepository
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory
    ) {
        $this->itemDataFactory = $itemDataFactory;
    }

    /**
     * Save additional data
     *
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $resultItem
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @throws CouldNotSaveException
     */
    public function afterSave(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderItemInterface $resultItem
    ) {
        /** @var \Magento\Sales\Api\Data\OrderInterface $resultItem */
        $resultItem = $this->saveOrderAdditionalData($resultItem);
        return $resultItem;
    }

    /**
     * Save additional data for item
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $item
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @throws CouldNotSaveException
     */
    protected function saveOrderAdditionalData(\Magento\Sales\Api\Data\OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if (
            null !== $extensionAttributes &&
            null !== $extensionAttributes->getFbAdditionalData()
        ) {
            
            $additionalData = current($extensionAttributes->getFbAdditionalData());
            try {
                $itemData=$this->itemDataFactory->create()->load($item->getItemId(), "item_id");
                $itemData->setItemId($item->getItemId());
                $itemData->setAdditionalData(serialize($additionalData));
                $itemData->save();                
            } catch (\Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for item: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $item;
    }

}
